@@ -292,11 +292,26 @@ euc2d <- function(coord1, coord2){
   as.numeric(round(sqrt(abs(coord1['x'] - coord2['x'])^2 + abs(coord1['y'] - coord2['y'])^2), 7))
 }
 
+# 2d Euclidean distance in segment space
+seg2d <- function(coord1, coord2){
+  as.numeric(round(sqrt(abs(coord1['MS'] - coord2['MS'])^2 + abs(coord1['LM'] - coord2['LM'])^2), 7))
+}
+
+# Achromatic contrast in segment space
+achroseg <- function(coord1, coord2){
+  as.numeric(abs(coord1['B'] - coord2['B']))
+}
+
 # Achromatic 'green' receptor contrast in the hexagon
 achrohex <- function(coord1, coord2){
   as.numeric(round(coord1['l'] - coord2['l'], 7))
 }
 
+# Achromatic contrast in cielab
+achrolab <- function(coord1, coord2){
+    as.numeric(abs(coord1['L'] - coord2['L']))
+}
+
 # 2d Euclidean distances in CIELAB
 lab2d <- function(coord1, coord2){
                      as.numeric(round(sqrt(abs(coord1['L'] - coord2['L'])^2 + abs(coord1['a'] - coord2['a'])^2 +
